@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
-/* $Id: items.c,v 1.18 2003/07/23 00:00:20 bradfitz Exp $ */
+/* $Id: items.c,v 1.21 2004/03/31 05:01:51 avva Exp $ */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -7,7 +7,6 @@
 #include <sys/socket.h>
 #include <sys/signal.h>
 #include <sys/resource.h>
-#include <sys/mman.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -17,7 +16,6 @@
 #include <errno.h>
 #include <time.h>
 #include <event.h>
-#include <malloc.h>
 #include <assert.h>
 
 #include "memcached.h"
@@ -57,6 +55,15 @@ item *item_alloc(char *key, int flags, time_t exptime, int nbytes) {
 
     it = slabs_alloc(ntotal);
     if (it == 0) {
+        int tries = 50;
+        item *search;
+
+        /* If requested to not push old items out of cache when memory runs out,
+         * we're out of luck at this point...
+         */
+
+        if (!settings.evict_to_free) return 0;
+
         /* 
          * try to get one off the right LRU 
          * don't necessariuly unlink the tail because it may be locked: refcount>0
@@ -64,9 +71,6 @@ item *item_alloc(char *key, int flags, time_t exptime, int nbytes) {
          * tries
          */
 
-        int tries = 50;
-        item *search;
-
         if (id > LARGEST_ID) return 0;
         if (tails[id]==0) return 0;
 
