@@ -13,7 +13,7 @@
  *      Anatoly Vorobey <mellon@pobox.com>
  *      Brad Fitzpatrick <brad@danga.com>
  *
- *  $Id: memcached.c,v 1.44 2003/12/01 21:18:22 avva Exp $
+ *  $Id: memcached.c,v 1.47 2004/04/24 20:00:59 avva Exp $
  */
 
 #include "config.h"
@@ -92,6 +92,7 @@ void settings_init(void) {
     settings.maxconns = 1024;         /* to limit connections-related memory to about 5MB */
     settings.verbose = 0;
     settings.oldest_live = 0;
+    settings.evict_to_free = 1;       /* push old items out of cache when memory runs out */
 }
 
 conn **freeconns;
@@ -1175,8 +1176,10 @@ void usage(void) {
     printf("-p <num>      port number to listen on\n");
     printf("-l <ip_addr>  interface to listen on, default is INDRR_ANY\n");
     printf("-d            run as a daemon\n");
+    printf("-r            maximize core file limit\n");
     printf("-u <username> assume identity of <username> (only when run as root)\n");
     printf("-m <num>      max memory to use for items in megabytes, default is 64 MB\n");
+    printf("-M            return error on memory exhausted (rather than removing items)\n");
     printf("-c <num>      max simultaneous connections, default is 1024\n");
     printf("-k            lock down all paged memory\n");
     printf("-v            verbose (print errors/warnings while in event loop)\n");
@@ -1265,15 +1268,17 @@ int main (int argc, char **argv) {
     struct in_addr addr;
     int lock_memory = 0;
     int daemonize = 0;
+    int maxcore = 0;
     char *username = 0;
     struct passwd *pw;
     struct sigaction sa;
+    struct rlimit rlim;
 
     /* init settings */
     settings_init();
 
     /* process arguments */
-    while ((c = getopt(argc, argv, "p:m:c:khivdl:u:")) != -1) {
+    while ((c = getopt(argc, argv, "p:m:Mc:khirvdl:u:")) != -1) {
         switch (c) {
         case 'p':
             settings.port = atoi(optarg);
@@ -1281,6 +1286,9 @@ int main (int argc, char **argv) {
         case 'm':
             settings.maxbytes = atoi(optarg)*1024*1024;
             break;
+        case 'M':
+            settings.evict_to_free = 0;
+            break;
         case 'c':
             settings.maxconns = atoi(optarg);
             break;
@@ -1307,6 +1315,9 @@ int main (int argc, char **argv) {
         case 'd':
             daemonize = 1;
             break;
+        case 'r':
+            maxcore = 1;
+            break;
         case 'u':
             username = optarg;
             break;
@@ -1316,6 +1327,53 @@ int main (int argc, char **argv) {
         }
     }
 
+    if (maxcore) {
+        struct rlimit rlim_new;
+        /* 
+         * First try raising to infinity; if that fails, try bringing
+         * the soft limit to the hard. 
+         */
+        if (getrlimit(RLIMIT_CORE, &rlim)==0) {
+            rlim_new.rlim_cur = rlim_new.rlim_max = RLIM_INFINITY;
+            if (setrlimit(RLIMIT_CORE, &rlim_new)!=0) {
+                /* failed. try raising just to the old max */
+                rlim_new.rlim_cur = rlim_new.rlim_max = 
+                    rlim.rlim_max;
+                (void) setrlimit(RLIMIT_CORE, &rlim_new);
+            }
+        }
+        /* 
+         * getrlimit again to see what we ended up with. Only fail if 
+         * the soft limit ends up 0, because then no core files will be 
+         * created at all.
+         */
+           
+        if ((getrlimit(RLIMIT_CORE, &rlim)!=0) || rlim.rlim_cur==0) {
+            fprintf(stderr, "failed to ensure corefile creation\n");
+            exit(1);
+        }
+    }
+                        
+    /* 
+     * If needed, increase rlimits to allow as many connections
+     * as needed.
+     */
+    
+    if (getrlimit(RLIMIT_NOFILE, &rlim) != 0) {
+        fprintf(stderr, "failed to getrlimit number of files\n");
+        exit(1);
+    } else {
+        int maxfiles = settings.maxconns;
+        if (rlim.rlim_cur < maxfiles) 
+            rlim.rlim_cur = maxfiles + 3;
+        if (rlim.rlim_max < rlim.rlim_cur)
+            rlim.rlim_max = rlim.rlim_cur;
+        if (setrlimit(RLIMIT_NOFILE, &rlim) != 0) {
+            fprintf(stderr, "failed to set rlimit for open files. Try running as root or requesting smaller maxconns value.\n");
+            exit(1);
+        }
+    }
+
     /* 
      * initialization order: first create the listening socket
      * (may need root on low ports), then drop root if needed,
@@ -1347,9 +1405,10 @@ int main (int argc, char **argv) {
     }
 
     /* daemonize if requested */
+    /* if we want to ensure our ability to dump core, don't chdir to / */
     if (daemonize) {
         int res;
-        res = daemon(0, settings.verbose);
+        res = daemon(maxcore, settings.verbose);
         if (res == -1) {
             fprintf(stderr, "failed to daemon() in order to daemonize\n");
             return 1;
